from qottoauth.api import (
    QottoAuthApi,
    QottoAuthApiError,
    QottoAuthGrapheneApi,
    QottoAuthTestApi,
)
from qottoauth.models import (
    Application,
    Permission,
    Namespace,
    Matching,
    Organization,
    Authorization,
    User,
    Member,
    Account,
    Cookie,
    Identity,
)
from qottoauth.service import (
    QottoAuthService,
)

__all__ = [
    'QottoAuthApi',
    'QottoAuthApiError',
    'QottoAuthGrapheneApi',
    'QottoAuthTestApi',
    'QottoAuthService',
    'Application',
    'Permission',
    'Namespace',
    'Matching',
    'Organization',
    'Authorization',
    'User',
    'Member',
    'Account',
    'Cookie',
    'Identity',
]



import logging
from typing import Any, Optional, Tuple

from eventy.trace_id import correlation_id_var, user_id_var
from gql import Client, gql
from gql.transport.exceptions import TransportQueryError, TransportServerError
from gql.transport.requests import RequestsHTTPTransport

from qottoauth.api.base import QottoAuthApi, QottoAuthApiError

__all__ = [
    'QottoAuthGrapheneApi',
]

logger = logging.getLogger(__name__)


class QottoAuthGrapheneApi(QottoAuthApi):
    def __init__(
            self,
            url: str,
    ):
        """
        :param url: Qotto Auth Backend URL
        """
        self.url = url

    def _graphql(
            self,
            query: str,
            variable_values: Optional[dict[str, Any]] = None,
    ):
        """
        Execute GraphQL query and return the result.

        :param query: The GraphQL query or mutation
        :param variable_values: The GraphQL params
        :return: The GraphQL response
        :raise QottoAuthError:
        """
        transport = RequestsHTTPTransport(
            url=self.url + '/graphql/',
            headers={
                'x-correlation-id': correlation_id_var.get(),
                'x-user-id': user_id_var.get(),
            },
        )
        client = Client(
            transport=transport,
        )
        try:
            result: dict[str, Any] = client.execute(
                gql(query),
                variable_values=variable_values,
            )
        except TransportQueryError as e:
            raise QottoAuthApiError('; '.join(map(str, e.errors)) if e.errors else 'Unknown error')
        except TransportServerError as e:
            logger.error(f"Transport error: {e.code} {e}")
            raise QottoAuthApiError(str(e))
        except ConnectionError as e:
            logger.error(f"Could not connect to server: {e}")
            raise QottoAuthApiError("Could not connect to Qotto Auth server.")

        if result.get('errors'):
            raise QottoAuthApiError(str(result.get('errors')))

        return result

    def query(
            self,
            name: str,
            body: str,
            variables: list[Tuple[str, str, Any]] = None,
    ):
        """
        Execute GraphQL query and return the result.

        :param name: GraphQL query name
        :param body: Body of the GraphQL query
        :param variables: List of (name, type, value) params
        :return: The GraphQL response
        """
        variables_types = ''
        variables_names = ''
        variable_values = {}
        query = ''
        if variables:
            variables_types = ', '.join(map(lambda x: f'${x[0]}: {x[1]}', variables))
            variable_values = dict(map(lambda x: (x[0], x[2]), variables))
            variables_names = ', '.join(map(lambda x: f'{x[0]}: ${x[0]}', variables))
            query += 'query ' + name + '(' + variables_types + ')' + ' {\n'
            query += '  ' + name + '(' + variables_names + ')'
        else:
            query += 'query ' + name + ' {\n'
            query += '  ' + name

        if body:
            query += ' {\n'
            query += '  ' + body + '\n'
            query += '   }\n'

        query += '}\n'
        return self._graphql(
            query=query,
            variable_values=variable_values,
        ).get(name)

    def mutation(
            self,
            name: str,
            body: str,
            input_name: str = 'input',
            input_type: str = None,
            input_value: dict[str, Any] = None,
    ):
        """
        Execute GraphQL mutation and return the result.

        :param name: GraphQL query name
        :param body: Body of the GraphQL query
        :param variables: List of (name, type, value) params
        :return: The GraphQL response
        """
        if input_type is None:
            input_type = name[:1].upper() + name[1:] + 'Input!'
        if input_value is None:
            input_value = {}
        variables = [
            (input_name, input_type, input_value),
        ]
        variables_types = ''
        variables_names = ''
        variable_values = {}
        if variables:
            variables_types = ', '.join(map(lambda x: f'${x[0]}: {x[1]}', variables))
            variable_values = dict(map(lambda x: (x[0], x[2]), variables))
            variables_names = ', '.join(map(lambda x: f'{x[0]}: ${x[0]}', variables))
        query = ''
        query += 'mutation ' + name + '(' + variables_types + ')' + ' {\n'
        query += '  ' + name + '(' + variables_names + ')' + ' {\n'
        query += '  ' + body + '\n'
        query += '   }\n'
        query += '}\n'
        return self._graphql(
            query=query,
            variable_values=variable_values,
        ).get(name)



from qottoauth.api.base import QottoAuthApi, QottoAuthApiError
from qottoauth.api.graphene import QottoAuthGrapheneApi
from qottoauth.api.test import QottoAuthTestApi

__all__ = [
    'QottoAuthApi',
    'QottoAuthApiError',
    'QottoAuthTestApi',
    'QottoAuthGrapheneApi',
]



import logging
from abc import ABC
from typing import Any, Tuple

__all__ = [
    'QottoAuthApi',
    'QottoAuthApiError',
]

logger = logging.getLogger(__name__)


class QottoAuthApiError(Exception):
    """
    Could not get a response from Qotto Auth API.
    """


class QottoAuthApi(ABC):
    def query(
            self,
            name: str,
            body: str,
            variables: list[Tuple[str, str, Any]] = None,
    ):
        """
        Execute GraphQL query and return the result.

        :param name: GraphQL query name
        :param body: Body of the GraphQL query
        :param variables: List of (name, type, value) params
        :return: The GraphQL response
        """
        raise NotImplementedError

    def mutation(
            self,
            name: str,
            body: str,
            input_name: str = 'input',
            input_type: str = None,
            input_value: dict[str, Any] = None,
    ):
        """
        Execute GraphQL mutation and return the result.

        :param name: GraphQL query name
        :param body: Body of the GraphQL query
        :param variables: List of (name, type, value) params
        :return: The GraphQL response
        """
        raise NotImplementedError



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['qottoauth', 'qottoauth.api']

package_data = \
{'': ['*']}

install_requires = \
['PyJWT>=2.0.0',
 'cryptography>=37.0.0',
 'eventy>=3.2.7a0',
 'gql[requests]>=2.0.0',
 'requests>=2.0.0']

setup_kwargs = {
    'name': 'qotto-auth-client',
    'version': '1.0.4a0',
    'description': 'Qotto/QottoAuthClient',
    'long_description': '# Qotto Auth Client\n\nThe python package `qotto-auth-client` is a client for the API `qotto-auth` which will soon be open sourced.\n\nIt allows to manage a scoped permission and authentication system.\n\nMore information coming soon...\n\n## Quickstart\n\nThe `QottoAuthApi` class allows to interact with a `qotto-auth` GraphQL server.\n\n```python\nfrom qottoauth import (\n    QottoAuthService,\n    QottoAuthApi,\n    QottoAuthGrapheneApi,\n    QottoAuthTestApi,\n)\n\n# Create a QottoAuthApi instance\napi: QottoAuthApi\nif TEST:\n    # for offline testing\n    api = QottoAuthTestApi()\nelse:\n    # qotto-auth server running\n    api = QottoAuthGrapheneApi(\n        url=QOTTO_AUTH_URL,\n    )\n\n# Create a QottoAuthService instance\nservice = QottoAuthService(\n    api=api,\n)\n\n# Register this application\nservice.register_application(\n    application_name=APPLICATION_NAME,\n    application_description=APPLICATION_DESCRIPTION,\n)\n\n# ...\n```\n\nYou can extend the `QottoAuthService` class to add your own business logic.\n\nIn the current version (1.0.1), `QottoAuthTestApi` does not provide all queries and mutations, so you might need to\nextend the class if you need to test beahviour depending on these queries and mutations.',
    'author': 'Qotto Dev Team',
    'author_email': 'dev@qotto.net',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)
